@@ -21,18 +21,24 @@
 #ifndef __GOSSIP_TIME_H__
 #define __GOSSIP_TIME_H__
 
+#define __USE_XOPEN
 #include <time.h>
-
 #include <glib.h>
 
+/* Note: Always in UTC. */
 typedef long gossip_time_t;
 
-gossip_time_t gossip_time_from_tm              (struct tm        *tm);
-struct tm *   gossip_time_to_tm                (gossip_time_t     t);
-gossip_time_t gossip_time_get_current          (void);
-gchar *       gossip_time_to_timestamp         (gossip_time_t     t);
-gchar *       gossip_time_to_timestamp_full    (gossip_time_t     t,
-						const gchar      *format);
+#define GOSSIP_TIME_FORMAT_DISPLAY_SHORT "%H:%M"
+#define GOSSIP_TIME_FORMAT_DISPLAY_LONG  "%a %d %b %Y"
+
+gossip_time_t  gossip_time_get_current     (void);
+gossip_time_t  gossip_time_parse           (const gchar   *str);
+gossip_time_t  gossip_time_parse_format    (const gchar *str,
+					    const gchar *format);
+gchar         *gossip_time_to_string_utc   (gossip_time_t  t,
+					    const gchar   *format);
+gchar         *gossip_time_to_string_local (gossip_time_t  t,
+					    const gchar   *format);
 
 #endif /* __GOSSIP_TIME_H__ */
 
