@@ -22,6 +22,7 @@
 #define __GOSSIP_CHAT_VIEW_H__
 
 #include <gtk/gtktextview.h>
+#include <gtk/gtktooltips.h>
 
 #include <libgossip/gossip-message.h>
 
@@ -32,24 +33,18 @@
 #define GOSSIP_IS_CHAT_VIEW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_CHAT_VIEW))
 #define GOSSIP_CHAT_VIEW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewClass))
 
-
 typedef struct _GossipChatView      GossipChatView;
 typedef struct _GossipChatViewClass GossipChatViewClass;
 typedef struct _GossipChatViewPriv  GossipChatViewPriv;
 
-
 struct _GossipChatView {
-	GtkTextView         parent;
-
-	GossipChatViewPriv *priv;
+	GtkTextView parent;
 };
 
-
 struct _GossipChatViewClass {
-	GtkTextViewClass    parent_class;
+	GtkTextViewClass parent_class;
 };
 
-
 typedef enum {
 	GOSSIP_SMILEY_NORMAL,       /*  :)   */
 	GOSSIP_SMILEY_WINK,         /*  ;)   */
@@ -72,39 +67,58 @@ typedef enum {
 	GOSSIP_SMILEY_OOOH,         /*  :x   */
 	GOSSIP_SMILEY_LOOKAWAY,     /*  *)   */
 	GOSSIP_SMILEY_BLUSH,        /*  *S   */
-	GOSSIP_SMILEY_COOLBIGSMILE, /*  8D   */ 
+	GOSSIP_SMILEY_COOLBIGSMILE, /*  8D   */
 	GOSSIP_SMILEY_ANGRY,        /*  :@   */
 	GOSSIP_SMILEY_BOSS,         /*  @)   */
 	GOSSIP_SMILEY_MONKEY,       /*  #)   */
 	GOSSIP_SMILEY_SILLY,        /*  O)   */
 	GOSSIP_SMILEY_SICK,         /*  +o(  */
-						
-	NUM_SMILEYS
-} GossipSmiley;
 
+	GOSSIP_SMILEY_COUNT
+} GossipSmiley;
 
 GType           gossip_chat_view_get_type                  (void) G_GNUC_CONST;
 GossipChatView *gossip_chat_view_new                       (void);
 void            gossip_chat_view_append_message_from_self  (GossipChatView *view,
 							    GossipMessage  *msg,
-							    GossipContact  *my_contact);
+							    GossipContact  *my_contact,
+							    GdkPixbuf      *avatar);
 void            gossip_chat_view_append_message_from_other (GossipChatView *view,
 							    GossipMessage  *msg,
-							    GossipContact  *my_contact);
-void            gossip_chat_view_append_invite             (GossipChatView *view,
-							    GossipMessage  *message);
+							    GossipContact  *my_contact,
+							    GdkPixbuf      *avatar);
 void            gossip_chat_view_append_event              (GossipChatView *view,
 							    const gchar    *str);
-void            gossip_chat_view_set_margin                (GossipChatView *view,
-							    gint            margin);
-void            gossip_chat_view_clear                     (GossipChatView *view);
+void            gossip_chat_view_append_invite             (GossipChatView *view,
+							    GossipMessage  *message);
+/* void            gossip_chat_view_set_margin                (GossipChatView *view, */
+/* 							    gint            margin); */
+void            gossip_chat_view_scroll                    (GossipChatView *view,
+							    gboolean        allow_scrolling);
 void            gossip_chat_view_scroll_down               (GossipChatView *view);
+
 gboolean        gossip_chat_view_get_selection_bounds      (GossipChatView *view,
 							    GtkTextIter    *start,
 							    GtkTextIter    *end);
+void            gossip_chat_view_clear                     (GossipChatView *view);
+void            gossip_chat_view_find                      (GossipChatView *view,
+							    const gchar    *search_criteria,
+							    gboolean        new_search);
+void            gossip_chat_view_highlight                 (GossipChatView *view,
+							    const gchar    *text);
 void            gossip_chat_view_copy_clipboard            (GossipChatView *view);
+
+gboolean        gossip_chat_view_get_irc_style             (GossipChatView *view);
 void            gossip_chat_view_set_irc_style             (GossipChatView *view,
 							    gboolean        irc_style);
-gboolean        gossip_chat_view_get_irc_style             (GossipChatView *view);
+void            gossip_chat_view_set_margin                (GossipChatView *view,
+							    gint            margin);
+GdkPixbuf *     gossip_chat_view_get_smiley_image          (GossipSmiley    smiley);
+const gchar *   gossip_chat_view_get_smiley_text           (GossipSmiley    smiley);
+GtkWidget *     gossip_chat_view_get_smiley_menu           (GCallback       callback,
+							    gpointer        user_data,
+							    GtkTooltips    *tooltips);
+void            gossip_chat_view_set_is_group_chat         (GossipChatView *view,
+							    gboolean        is_group_chat);
 
 #endif /* __GOSSIP_CHAT_VIEW_H__ */
