@@ -26,7 +26,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtktextview.h>
-#include <gtk/gtkmessagedialog.h> 
+#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtksizegroup.h>
 #include <glade/glade.h>
 
@@ -61,6 +61,18 @@ void       gossip_glade_setup_size_group            (GladeXML            *gui,
 /* Dialogs */
 gchar *    gossip_password_dialog_run               (GossipAccount       *account,
 						     GtkWindow           *parent);
+gboolean   gossip_hint_dialog_show                  (const gchar         *conf_path,
+						     const gchar         *message1,
+						     const gchar         *message2,
+						     GtkWindow           *parent,
+						     GFunc                func,
+						     gpointer             user_data);
+gboolean   gossip_hint_show                         (const gchar         *conf_path,
+						     const gchar         *message1,
+						     const gchar         *message2,
+						     GtkWindow           *parent,
+						     GFunc                func,
+						     gpointer             user_data);
 
 /* Pixbufs */
 GdkPixbuf *gossip_pixbuf_from_stock                 (const gchar         *stock,
@@ -91,8 +103,10 @@ GdkPixbuf *gossip_pixbuf_avatar_from_vcard_scaled   (GossipVCard         *vcard,
 						     GtkIconSize          size);
 GdkPixbuf *gossip_pixbuf_avatar_from_contact        (GossipContact       *contact);
 GdkPixbuf *gossip_pixbuf_avatar_from_contact_scaled (GossipContact       *contact,
-						     GtkIconSize          size);
+						     gint                 width,
+						     gint                 height);
 
+/* Text view */
 gboolean   gossip_text_iter_forward_search          (const GtkTextIter   *iter,
 						     const gchar         *str,
 						     GtkTextIter         *match_start,
@@ -101,6 +115,16 @@ gboolean   gossip_text_iter_forward_search          (const GtkTextIter   *iter,
 
 /* Windows */
 gboolean   gossip_window_get_is_visible             (GtkWindow           *window);
-void       gossip_window_present                    (GtkWindow           *window);
+void       gossip_window_present                    (GtkWindow           *window,
+						     gboolean             steal_focus);
+void       gossip_window_set_default_icon_name      (const gchar         *name);
+
+void       gossip_url_show                          (const char          *url);
+void       gossip_help_show                         (void);
+
+GtkWidget *gossip_link_button_new                   (const gchar         *url,
+						     const gchar         *title);
+
+void       gossip_request_user_attention            (void);
 
 #endif /*  __GOSSIP_UI_UTILS_H__ */
